#!/usr/bin/env python3
"""
    Resources
    =========

    The following constitutes the base-logic of CIJOE:

    * cijoe.core.command (Cijoe)
    * cijoe.core.transport (Transport, Local, SSH)
    * cijoe.core.misc (As the name suggests; various helper-functions)
    * cijoe.cli (Command-Line Tool and utilization of the above for workflow execution)

    Everything else, literally everything, is implemented as dynamically
    collectable and loadable resources. That is, cijoe
    configuration-files/scripts/workflows/templates, and auxiliary files.

    The base-representation of these resources are the cijoe.core.resources.Resource
    class, with content-specific subclasses (Config, Script, and Workflow).

    These resources are collected from installed and locally available Packages, as well
    as for path by the cijoe.core.resources.Collector.

    The Collector is a SingleTon, since it is used extensively everywhere and the tasks
    of doing collection can be somewhat time-consuming.

    Intended usage
    --------------

    From Python::

        from cijoe.core.resources import get_resources
        resources = get_resources()

    From command-line::

        cijoe --resources
"""
import ast
import importlib
import inspect
import logging as log
import os
import pkgutil
import re
import sys
from importlib.machinery import SourceFileLoader
from pathlib import Path

try:
    from importlib.resources import files as importlib_files
except ImportError:
    from importlib_resources import files as importlib_files

import jinja2
import yaml

import cijoe

try:
    import tomllib
except ModuleNotFoundError:
    import tomli as tomllib
import tomli_w


def dict_from_yamlfile(path: Path):
    """Returns content of yamlfile at 'path' as dict and {} on empty document."""

    with path.open("rb") as yamlfile:
        return yaml.safe_load(yamlfile) or {}


def dict_from_tomlfile(path: Path):
    """Returns content of tomfile at 'path' as dict and {} on empty document."""

    with path.open("rb") as tomlfile:
        return tomllib.load(tomlfile) or {}


def dict_to_tomlfile(data, path: Path):
    """Dumps the given 'data' to the given 'path'"""

    with path.open("wb") as tomlfile:
        return tomli_w.dump(data, tomlfile)


def default_context(config=None, resources=None):
    """Return a default context for dict-substitution"""

    if resources is None:
        resources = get_resources()

    return {
        "local": {
            "env": os.environ,
        },
        "config": config.options if config else {},
        "resources": resources,
    }


def dict_substitute(topic: dict, context: dict) -> list:
    """Traverse the given 'topic' replacing {{ foo.bar }} entities with ctx. values"""

    errors = []

    jinja_env = jinja2.Environment(undefined=jinja2.StrictUndefined)
    for key, value in topic.items():
        try:
            if isinstance(value, str):
                topic[key] = jinja_env.from_string(value).render(context)
            elif isinstance(value, list) and all(
                isinstance(line, str) for line in value
            ):
                topic[key] = [
                    jinja_env.from_string(line).render(context) for line in value
                ]
            elif isinstance(value, list) and all(
                isinstance(entity, dict) for entity in value
            ):
                for entity in value:
                    errors += dict_substitute(entity, context)
            elif isinstance(value, dict):
                errors += dict_substitute(value, context)
        except jinja2.exceptions.UndefinedError as exc:
            errors.append(f"Substitution-error: {exc}")

    return errors


class Resource(object):
    """Base representation of a Resource"""

    def __init__(self, path: Path, pkg=None):
        self.path = path.resolve()
        self.pkg = pkg

        self.path = path
        self.content = None

        prefix = ".".join(pkg.name.split(".")[1:-1]) + "." if pkg else ""

        self.ident = f"{prefix}{self.path.stem}"

    def __repr__(self):
        return str(self.path)

    def content_from_file(self):
        """Load resource-content from 'self.path'"""

        with self.path.open("r") as resource_file:
            self.content = resource_file.read()


class Config(Resource):
    """
    Encapsulation of a CIJOE config-file, e.g. 'default-config.toml'

    ivar: options: dict of configuration options populated by load() / from_path()
    """

    SUFFIX = ".toml"

    def __init__(self, path: Path, pkg=None):
        super().__init__(path, pkg)

        self.options = {}

    def load(self):
        """Populates self.options on success. Returns a list of errors otherwise"""

        # config_dict = dict_from_yamlfile(self.path)
        config_dict = dict_from_tomlfile(self.path)

        errors = dict_substitute(config_dict, default_context())
        if errors:
            return errors

        self.options = config_dict
        return []

    @staticmethod
    def from_path(path, pkg=None):
        """Instantiate a Config from path, returning None on error"""

        path = Path(path).resolve()
        if not path.exists():
            return None

        config = Config(path, pkg)
        errors = config.load()
        if errors:
            return None

        return config


class Script(Resource):
    """Script representation and encapsulation"""

    SUFFIX = ".py"
    NAMING_CONVENTION = ["worklet_entry", "script_entry", "main"]

    def __init__(self, path, pkg=None):
        super().__init__(path, pkg)

        self.func = None
        self.mod = None
        self.mod_name = None

    def content_has_script_func(self):
        """Checks whether the resource-content has the script entry-function"""

        try:
            tree = ast.parse(self.content)
        except SyntaxError:
            return False

        for node in [x for x in ast.walk(tree) if isinstance(x, ast.FunctionDef)]:
            if node.name not in Script.NAMING_CONVENTION:
                log.debug(f"skipping; invalid entry-name({node.name})")
                continue

            argnames = [arg.arg for arg in node.args.args]
            if argnames != ["args", "cijoe", "step"]:
                log.debug(f"skipping; invalid argnames({argnames})")
                continue

            return True

        return False

    def load(self):
        """Loads the module and the script-entry function"""

        if self.func:
            return []

        if not self.content:
            self.content_from_file()

        if not self.content_has_script_func():
            return ["Missing script_entry() function in ast"]

        mod_name = str(Path(self.path).resolve().stem)
        mod_path = str(Path(self.path).resolve())

        parent_path = str(Path(self.path).resolve().parent)
        if parent_path not in sys.path:
            sys.path.append(parent_path)

        spec = importlib.util.spec_from_loader(
            mod_name, SourceFileLoader(mod_name, mod_path)
        )
        mod = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(mod)

        # mod = SourceFileLoader("", str(self.path)).load_module()
        for function_name, function in inspect.getmembers(mod, inspect.isfunction):
            if function_name not in Script.NAMING_CONVENTION:
                continue

            self.mod = mod
            self.mod_name = mod_name
            self.func = function
            return []

        return ["Missing script_entry() function in loaded module"]


class Workflow(Resource):
    SUFFIX = ".yaml"
    STATE_FILENAME = "workflow.state"
    STATE = {
        "doc": "",
        "tag": "",
        "config": {},
        "steps": [],
        "status": {
            "skipped": 0,
            "failed": 0,
            "passed": 0,
            "elapsed": 0.0,
            "started": 0.0,
        },
    }

    def __init__(self, path, pkg=None):
        super().__init__(path, pkg)

        self.state = None
        self.config = None

    def state_dump(self, path):
        """Dump the current workflow-state to yaml-file"""

        with path.open("w+") as state_file:
            yaml.dump(self.state, state_file)

    @staticmethod
    def dict_normalize(topic: dict):
        """Normalize the workflow-dict, transformation of the 'run' shorthand"""

        errors = []

        if "steps" not in topic:
            errors.append("Missing required top-level key: 'steps'")
            return errors

        for step in topic["steps"]:
            if step["name"].endswith(".py"):
                errors.append(
                    "Illegal step name: Suffix .py reserved for Python scripts"
                )
                return errors

            if "run" not in step.keys():
                continue

            step["uses"] = "core.cmdrunner"
            step["with"] = {"commands": step["run"].splitlines()}

            del step["run"]

        return errors

    @staticmethod
    def dict_lint(topic: dict):
        """Returns a list of integrity-errors for the given workflow-dict(topic)"""

        resources = get_resources()

        errors = []

        for top in set(topic.keys()) - set(["doc", "config", "steps"]):
            errors.append(f"Unsupported top-level key: '{top}'")
            return errors
        for top in ["doc", "steps"]:
            if top not in topic:
                errors.append(f"Missing required top-level key: '{top}'")
                return errors

        step_names = []
        duplicate_names = []
        for step in topic["steps"]:
            if step["name"] in step_names:
                duplicate_names.append(step["name"])
            step_names.append(step["name"])

        if len(duplicate_names):
            errors.append(f"Duplicate step-names: {duplicate_names}")
            return errors

        valid = set(["name", "uses", "with"])
        required = set(["name", "uses"])

        for nr, step in enumerate(topic["steps"], 1):
            keys = set(step.keys())

            missing = required - keys
            if missing:
                errors.append(f"Invalid step({nr}); required key(s): {missing}")
                continue

            if not re.match(r"^([a-zA-Z][a-zA-Z0-9\.\-_]*)", step["name"]):
                errors.append(f"Invalid step({nr}); invalid chars in 'name'")
                continue

            unsupported = keys - valid
            if unsupported:
                errors.append(f"Invalid step({nr}); unsupported keys({unsupported})")
                continue

            if step["uses"] not in resources["scripts"]:
                errors.append(
                    f"Invalid step({nr}); unknown resource: script({step['uses']})"
                )
                continue

        return errors

    def load(self, config: Config, extra_steps: list = []):
        """
        Load the workflow-yamlfile, normalize it, lint it, substitute, then construct
        the object properties
        """

        errors = []

        if self.state:
            return errors

        workflow_dict = dict_from_yamlfile(self.path)
        workflow_dict["steps"] += extra_steps

        errors += Workflow.dict_normalize(workflow_dict)
        if errors:
            return errors

        errors += Workflow.dict_lint(workflow_dict)
        if errors:
            return errors

        errors += dict_substitute(workflow_dict, default_context(config))
        if errors:
            return errors

        state = Workflow.STATE.copy()
        state["doc"] = workflow_dict.get("doc")
        state["config"] = workflow_dict.get("config", {})
        for nr, step in enumerate(workflow_dict["steps"], 1):
            step["nr"] = nr
            step["status"] = {
                "skipped": 0,
                "passed": 0,
                "failed": 0,
                "elapsed": 0.0,
                "started": 0.0,
            }
            step["id"] = f"{nr:03}_{step['name']}"

            state["steps"].append(step)

        self.state = state

        return errors


class Collector(object):
    """Collects resources from installed packages and the current working directory"""

    RESOURCES = [
        ("configs", Config.SUFFIX),
        ("perf_reqs", ".perfreq"),
        ("templates", ".html"),
        ("workflows", Workflow.SUFFIX),
        ("scripts", Script.SUFFIX),
        ("auxiliary", ".*"),
    ]
    IGNORE = ["__init__.py", "__pycache__", "setup.py"]

    def __new__(cls):
        if not hasattr(cls, "instance"):
            cls.instance = super(Collector, cls).__new__(cls)
            cls.instance.reset()
        return cls.instance

    def __process_candidate(self, candidate: Path, category: str, pkg):
        """Inserts the given candidate"""

        if category == "scripts":
            resource = Script(candidate, pkg)
            resource.content_from_file()

            if not resource.content_has_script_func():
                category = "auxiliary"
        elif category == "configs":
            resource = Config(candidate, pkg)
        elif category == "workflows":
            resource = Workflow(candidate, pkg)
        else:
            resource = Resource(candidate, pkg)

        self.resources[category][resource.ident] = resource

    def reset(self):
        self.resources = {category: {} for category, _ in Collector.RESOURCES}
        self.is_done = False

    def collect_from_path(self, path=None, max_depth=2):
        """Collects non-packaged scripts from the given 'path'"""

        path = Path(path).resolve() if path else Path.cwd().resolve()

        base = len(str(path).split(os.sep))

        for candidate in list(path.glob("*")) + list(path.glob("*/*")):
            level = len(str(candidate).split(os.sep))
            if max_depth and level > base + max_depth:
                continue

            if candidate.parent.stem == "auxiliary":
                self.__process_candidate(candidate, "auxiliary", None)
                continue

            for category, suffix in Collector.RESOURCES:
                if candidate.name in Collector.IGNORE:
                    continue
                if candidate.suffix != suffix:
                    continue

                self.__process_candidate(candidate, category, None)

    def collect_from_packages(self, path=None, prefix=None):
        """Collect resources from CIJOE packages at the given 'path'"""

        if prefix is None:
            prefix = ""

        for pkg in pkgutil.walk_packages(path, prefix):
            comp = pkg.name.split(".")[1:]  # drop the 'cijoe.' prefix
            if not (
                pkg.ispkg
                and any(cat in comp for cat, _ in Collector.RESOURCES)
                and len(comp) == 2
            ):  # skip non-resource packages
                continue

            _, category = comp
            for candidate in importlib_files(f"{pkg.name}").iterdir():
                if candidate.name in Collector.IGNORE:
                    continue

                self.__process_candidate(candidate, category, pkg)

    def collect(self, paths=[]):
        """Collect from all implemented resource "sources" """

        if self.is_done:
            return

        self.collect_from_packages(cijoe.__path__, "cijoe.")
        self.collect_from_path()

        for path in paths:
            self.collect_from_path(path, 0)

        cwd = Path().cwd() / ".cijoe"
        if cwd.exists():
            self.collect_from_path(cwd)

        home = Path().home() / ".cijoe"
        if home.exists():
            self.collect_from_path(home)

        self.is_done = True


def get_resources(paths=[]):
    """Returns resources collected by Collector"""

    collector = Collector()
    collector.collect(paths)

    return collector.resources



"""
repository_prep
===============

For every key in the configuration which has a subkey named "repository", then
following is done:

 * git clone repository.remote                   # if ! exists(repository.path)
 * git checkout [repository.branch,repository.tag] # if repository.{branch,tag}
 * git pull --rebase                               # if repository.branch
 * git status

The intended usage of this script is to prepare a repositories in a recently
provisioned system. Such as a done by 'qemu.provision'.

Configuration
-------------

Ensure that the "repository" has sensible values for:

* remote: url of the repository to clone

* branch: name of the branch to check out and rebase
* tag: name of the tag to check out

* run_local: Optionally, set 'run_local' to True, in case the repos should just be
  checked out locally, instead of on the remote.

Retargetable: True
------------------
"""
import errno
import logging as log
from pathlib import Path


def main(args, cijoe, step):
    """Clone, checkout branch and pull"""

    err, _ = cijoe.run("git --version")
    if err:
        log.error("Looks like git is not available")
        return err

    for repos in [
        r["repository"] for r in cijoe.config.options.values() if "repository" in r
    ]:
        run = cijoe.run_local if repos.get("run_local", False) else cijoe.run

        repos_root = Path(repos["path"]).parent

        err, _ = run(f"mkdir -p {repos_root}")
        if err:
            log.error("failed creating repos_root({repos_root}; giving up")
            return err

        err, _ = run(
            f"[ ! -d {repos['path']} ] &&"
            f" git clone {repos['remote']} {repos['path']} --recursive"
        )
        if err:
            log.info("either already cloned or failed cloning; continuing optimisticly")

        err, _ = run("git fetch --all", cwd=repos["path"])
        if err:
            log.info("fetching failed; continuing optimisticly")

        do_checkout = repos.get("branch", repos.get("tag", None))
        if do_checkout:
            err, _ = run(f"git checkout {do_checkout}", cwd=repos["path"])
            if err:
                log.error("Failed checking out; giving up")
                return err
        else:
            log.info("no 'branch' nor 'tag' key; skipping checkout")

        if "branch" in repos.keys():
            err, _ = run("git pull --rebase", cwd=repos["path"])
            if err:
                log.error("failed pulling; giving up")
                return err

        err, _ = run("git submodule update --init --recursive", cwd=repos["path"])
        if err:
            log.info("Updating submodules failed; continuin optimisticly")

        err, _ = run("git status", cwd=repos["path"])
        if err:
            log.error("failed 'git status'; giving up")
            return err

    return 0



"""
    fio-wrapper
    ===========

    The intent is to be able to do tie fio into testing infra and to have a means of
    ensuring that "benchmark" can be performed in a reproducible manner.

    It does so by constructing an transforming a dictionary of parameters and
    environment variables. This is done via:

    * fio(cijoe, parameters="", env={}): invoke fio as defined in cijoe.config.options

    * setup_job(param, env, job_name): add parameters JOBS accoring to name

    * setup_ioengine(param, env, ...): add parameters and env. according to ioengine

    * setup_output(param, env, ...): add parameters for output format and file

    * fio_fancy(cijoe, ...): construct parameters using setup_* helpers, then call fio()

    Then **chaining** of the parameter construction is done, such that it is possible to
    control certain job-parameters depending on the device, I/O engine, and I/O engine
    options.
    For example, we might wish to run fio with ``--direct=1``, however, when sending
    that same job to e.g. ``/dev/ng0n1`` (the char-device encapsulating an NVMe
    namespace), then ``--direct=1`` is invalid.
    Such a transformation is then done when seeing that the device is an NVMe chardev.

    config
    ------

    fio.bin

    fio.engines

    retargtable: true
    -----------------
"""
import errno
import json
import logging as log
import os
from pathlib import Path

JOBS = {
    "compare": {
        "name": "compare",
        "rw": "randread",
        "size": "12G",
        "iodepth": "1",
        "bs": "4K",
        "direct": "1",
        "thread": "1",
        "time_based": "1",
        "runtime": "7",
        "ramp_time": "3",
        "norandommap": "1",
        "allow_file_create": "0",
    },
    "verify": {
        "name": "verify",
        "rw": "randwrite",
        "size": "1G",
        "iodepth": "16",
        "bs": "4K",
        "direct": "1",
        "thread": "1",
        "verify": "crc32c",
        "allow_file_create": "0",
    },
    "zoned": {
        "name": "zoned",
        "zonemode": "zbd",
        "rw": "write",
        "size": "1G",
        "iodepth": "1",
        "bs": "4K",
        "direct": "1",
        "thread": "1",
        "ramp_time": "1",
        "norandommap": "1",
        "verify": "crc32c",
        "allow_file_create": "0",
    },
}


def setup_job(param, env, job_name):
    """Setup parameters for the fio-job based on the templates in JOBS"""

    param.update(JOBS[job_name])


def setup_ioengine(param, env, engine_name, cijoe, device, xnvme_opts, spdk_opts):
    """
    Setup dictionaries of environment variables and parameters for the given I/O engine

    env: environments variables needed by the ioengine
    param: parameters for the ioengine (param)

    @returns env, param
    """

    engine = (
        cijoe.config.options.get("fio", {}).get("engines", {}).get(engine_name, None)
    )
    if engine is None:
        log.error(f"fio.engine({engine_name}) not in configuration")
        return False

    # disable '--direct' for char-devices
    if "cdev" in device["labels"]:
        param["direct"] = "0"

    # setup the --ioengine flag and possibly the LD_PRELOAD environment variable
    if engine["type"] == "builtin":
        param["ioengine"] = engine_name
    elif engine["type"] == "external_dynamic":
        param["ioengine"] = f"external:{ engine['path'] }"
    elif engine["type"] == "external_preload":
        param["ioengine"] = engine_name
        env["LD_PRELOAD"] = engine["path"]
    else:
        log.error(f"Configuration has invalid engine.type({ engine['type'] })")
        return False

    # setup 'xnvme' specific options
    if engine_name == "xnvme":
        if "be" in xnvme_opts:
            param["xnvme_be"] = xnvme_opts["be"]
        if "async" in xnvme_opts:
            param["xnvme_async"] = xnvme_opts["async"]
        if "sync" in xnvme_opts:
            param["xnvme_sync"] = xnvme_opts["sync"]
        if "admin" in xnvme_opts:
            param["xnvme_admin"] = xnvme_opts["admin"]
        if "nsid" in device:
            param["xnvme_dev_nsid"] = device["nsid"]
        if "mem" in xnvme_opts:
            param["xnvme_mem"] = xnvme_opts["mem"]

        if any(label in ["pcie", "fabrics"] for label in device["labels"]):
            param["filename"] = device["uri"].replace(":", r"\:")
        else:
            param["filename"] = device["uri"]
    elif engine_name == "spdk":
        param["filename"] = " ".join(
            [
                "trtype=PCIe",
                f"traddr={device['uri']}",
                f"ns={device['nsid']}",
            ]
        )
    elif engine_name == "spdk_bdev":
        spdk_conf = {
            "subsystems": [
                {
                    "subsystem": "bdev",
                    "config": [
                        {
                            "params": spdk_opts["params"],
                            "method": f"bdev_{spdk_opts['bdev']}_create",
                        }
                    ],
                }
            ]
        }
        conf_filestem = "_".join(
            [
                "spdk",
                "bdev",
                spdk_opts["bdev"],
                spdk_opts["params"].get("io_mechanism", "foo"),
            ]
        )
        spdk_conf_path = Path(f"/tmp/{conf_filestem}.conf")
        with spdk_conf_path.open("w") as scpath:
            json.dump(spdk_conf, scpath)

        cijoe.put(spdk_conf_path, spdk_conf_path)

        param["spdk_conf"] = f"{spdk_conf_path}"
        param["filename"] = f"{spdk_opts['params']['name']}"
    else:
        param["filename"] = device["uri"]


def setup_output(param, env, output_fpath):
    """
    Sets up fio to store output in 'output_fpath' in formated in JSON as well as normal
    output. For postprocessing the json-document needs to be extracted as it is
    intermixed without non-JSON content.
    """

    param["output-format"] = "normal,json"
    param["output"] = f"{output_fpath}"


def fio(cijoe, parameters="", env={}):
    """
    Invoke 'fio' using binary at `'cijoe.config.options.fio.bin`', with the given
    parameters and environment variables (``env``)

    @returns err, state
    """

    return cijoe.run(f"{cijoe.config.options['fio']['bin']} {parameters}", env=env)


def fio_fancy(
    cijoe,
    output_fpath,
    jobname,
    engine_name,
    device,
    xnvme_opts={},
    spdk_opts={},
    aux={},
    env={},
):
    """
    Run fio using helpers for parameter setup for io-engines, store fio output with
    json, collect the output as artifact

    @returns err, state
    """

    param, job_env = {}, {}  # Setup parameters and env.
    setup_job(param, job_env, jobname)
    setup_ioengine(param, job_env, engine_name, cijoe, device, xnvme_opts, spdk_opts)
    setup_output(param, job_env, output_fpath)

    param.update(aux)

    parameters = " ".join([f'--{key}="{val}"' for key, val in param.items()])

    err, _ = cijoe.run(f"rm {output_fpath}")  # Avoid getting old data...
    if err:
        log.info(f"failed removing '{output_fpath}'")

    err, state = fio(cijoe, parameters, env=job_env | env)

    cijoe.run(f"cat {output_fpath}")  # Get the output in runlog

    artifacts = state.output_dpath / "artifacts"  # Collect output as artifact
    os.makedirs(artifacts, exist_ok=True)
    cijoe.get(str(output_fpath), str(artifacts / output_fpath.name))

    return err, state


def dict_from_fio_output_file(fpath: Path) -> dict:
    """
    The JSON output produced by fio comes mixed with non-JSON output, this function
    attempts to extract the JSON part, parse it and return the JSON-document as a dict.

    On error, an empty dict is returned.
    """

    lines = []

    with fpath.open() as fio_output:
        do_append = False
        for line in fio_output:
            if line.startswith("{"):
                do_append = True

            if do_append:
                lines.append(line)

            if line.startswith("}"):
                break
    try:
        return json.loads("".join(lines))
    except json.decoder.JSONDecodeError:
        return {}



import filecmp
import os
import tempfile

from cijoe.core.misc import ENCODING


def test_push_pull(cijoe):
    """Create a file, fill it, push it, pull it, and compare"""

    with tempfile.NamedTemporaryFile(
        encoding=ENCODING, mode="a", delete=False
    ) as test_file:
        test_file.write("".join([chr(65 + (i % 24)) for i in range(4096)]))
        test_file.flush()

        assert cijoe.put(test_file.name, "foo"), "Failed push()"

        assert cijoe.get("foo", "bar"), "Failed pull()"

        pulled = os.path.join(cijoe.output_path, cijoe.output_ident, "bar")

        assert filecmp.cmp(test_file.name, pulled, shallow=False), "Failed cmp()"



#!/usr/bin/env python3
"""
Create ``boot.img`` from a bootable disk-image
==============================================

Initialize the guest's boot image (``boot.img``) using an existing bootable
disk-image at ``qemu.guest.init_using_bootimage.img``. When the image-file is not
available, then it will be downloaded from ``qemu.guest.init_using_bootimage.url``.

Config::

    [qemu.guest.init_using_bootimage]
    url = # URL pointing to download location of the bootable disk-image
    img = # Absolute path to disk-image file"

Retargetable: False
-------------------
"""
from cijoe.qemu.wrapper import Guest


def main(args, cijoe, step):
    """Provision using an existing boot image"""

    guest = Guest(cijoe, cijoe.config)

    return guest.init_using_bootimage()



def test_hello():
    assert True


def test_default(cijoe):
    err, state = cijoe.run("ls -lh")

    assert err == 0


def test_default_again(cijoe):
    err, state = cijoe.run("ls")

    assert err == 0



"""
Setup a batch of self-hosted GitHub Action Runners

* This script will register 'gha.runner.count' number of GitHub Action runners
* Each runner will have 'gha.runner.labels' associated
* Each runner will be named e.g. 'gha.runner.nameprefix-12'
* The runners will live in 'gha.runner.home / runners / gha.runner.nameprefix-12'

Ensure that the system has docker, qemu, and a user 'ghar' exists::

    adduser ghar
    usermod -aG sudo ghar
    usermod -aG libvirt ghar
    usermod -aG docker ghar

Before running, then the environment variable::

    GHA_RUNNER_TOKEN

Below is an example of the configuration items that this script uses::

    [gha.runner]
    token = "your_secret_runner_token_set_here_or_env"
    url.repository = "https://github.com/safl/spdk-community-ci"
    home = "/opt/gha"
    nameprefix = "qemu-host"
    count = 10
    labels = ["qemuhost"]
"""

import logging as log
import os
import sys
from pathlib import Path

from cijoe.cli.cli import cli_interface


def main(args, cijoe, step):
    runner = cijoe.config.options.get("gha", {}).get("runner", {})

    url = runner.get("url", {}).get("repository", None)
    user = runner.get("user", None)
    home = runner.get("home", None)
    count = runner.get("count", None)
    labels = runner.get("labels", None)
    nameprefix = runner.get("nameprefix", None)
    token = os.getenv("GHA_RUNNER_TOKEN", runner.get("token", None))

    if None in [url, home, count, labels, nameprefix, token]:
        token = "your_secret_runner_token_set_here_or_env"
        log.error(f"missing or invalid config gha.runner({runner})")
        return 1

    labels = ",".join(labels)
    for number in range(int(count)):
        name = f"{nameprefix}{number:02d}"
        rdir = f"{home}/runners/{name}"

        for cmd in [f"mkdir {rdir}/", f"cp -r {home}/ghar/. {rdir}/."]:
            err, _ = cijoe.run(cmd)
            if err:
                log.error(f"failed copying runner err({err})")
                return err

        for cmd in [
            f"chown -R {user}:{user} .",
            f"./config.sh --unattended --replace "
            f"--url {url} --token {token} --labels {labels} --name {name}",
            f"sudo ./svc.sh install {user}",
            f"sudo ./svc.sh start",
            f"sudo ./svc.sh status",
        ]:
            err, _ = cijoe.run(cmd, cwd=rdir)
            if err:
                log.error(f"cmd({cmd}), err({err})")
                return err

    return err


if __name__ == "__main__":
    cli_interface(__file__)



from cijoe.cli.cli import cli_interface


def main(args, cijoe, step):
    err, state = cijoe.run("echo 'Hello World!'")
    if "Hello" not in state.output():
        return 1
    return err


if __name__ == "__main__":
    cli_interface(__file__)
